@@ -56,7 +56,7 @@ class Converter(object):
     _FORMAT_INFO = {
         Format.JSON: {
             'title': 'JSON',
-            'description': 'List of items in raw JSON format stored in one JSON file. Use to export both the data and the annotations for a dataset.',
+            'description': "List of items in raw JSON format stored in one JSON file. Use to export both the data and the annotations for a dataset. It's Label Studio Common Format",
             'link': 'https://labelstud.io/guide/export.html#JSON'
         },
         Format.JSON_MIN: {
@@ -82,13 +82,13 @@ class Converter(object):
         },
         Format.COCO: {
             'title': 'COCO',
-            'description': 'Popular machine learning format used by the COCO dataset for object detection and image segmentation tasks.',
+            'description': 'Popular machine learning format used by the COCO dataset for object detection and image segmentation tasks with polygons and rectangles.',
             'link': 'https://labelstud.io/guide/export.html#COCO',
             'tags': ['image segmentation', 'object detection']
         },
         Format.VOC: {
             'title': 'Pascal VOC XML',
-            'description': 'Popular XML-formatted task data used for object detection and image segmentation tasks.',
+            'description': 'Popular XML format used for object detection and polygon image segmentation tasks.',
             'link': 'https://labelstud.io/guide/export.html#Pascal-VOC-XML',
             'tags': ['image segmentation', 'object detection']
         },
@@ -134,6 +134,7 @@ class Converter(object):
     def convert(self, input_data, output_data, format, is_dir=True, **kwargs):
         if isinstance(format, str):
             format = Format.from_string(format)
+            
         if format == Format.JSON:
             self.convert_to_json(input_data, output_data, is_dir=is_dir)
         elif format == Format.JSON_MIN:
@@ -155,12 +156,16 @@ class Converter(object):
             image_dir = kwargs.get('image_dir')
             self.convert_to_voc(input_data, output_data, output_image_dir=image_dir, is_dir=is_dir)
         elif format == Format.BRUSH_TO_NUMPY:
-            brush.convert_task_dir(input_data, output_data, out_format='numpy')
+            items = self.iter_from_dir(input_data) if is_dir else self.iter_from_json_file(input_data)
+            brush.convert_task_dir(items, output_data, out_format='numpy')
         elif format == Format.BRUSH_TO_PNG:
-            brush.convert_task_dir(input_data, output_data, out_format='png')
+            items = self.iter_from_dir(input_data) if is_dir else self.iter_from_json_file(input_data)
+            brush.convert_task_dir(items, output_data, out_format='png')
         elif format == Format.ASR_MANIFEST:
             items = self.iter_from_dir(input_data) if is_dir else self.iter_from_json_file(input_data)
-            convert_to_asr_json_manifest(items, output_data, data_key=self._data_keys[0], project_dir=self.project_dir)
+            convert_to_asr_json_manifest(
+                items, output_data, data_key=self._data_keys[0], project_dir=self.project_dir,
+                upload_dir=self.upload_dir)
 
     def _get_data_keys_and_output_tags(self, output_tags=None):
         data_keys = set()
@@ -201,7 +206,6 @@ class Converter(object):
         if not ('Image' in input_tag_types and ('BrushLabels' in output_tag_types or 'brushlabels' in output_tag_types)):
             all_formats.remove(Format.BRUSH_TO_NUMPY.name)
             all_formats.remove(Format.BRUSH_TO_PNG.name)
-
         if not (('Audio' in input_tag_types or 'AudioPlus' in input_tag_types) and 'TextArea' in output_tag_types):
             all_formats.remove(Format.ASR_MANIFEST.name)
 
@@ -220,51 +224,65 @@ class Converter(object):
                     yield item
 
     def iter_from_json_file(self, json_file):
+        """ Extract annotation results from json file
+
+        param json_file: path to task list or dict with annotations
+        """
         with io.open(json_file, encoding='utf8') as f:
             data = json.load(f)
+            # one task
             if isinstance(data, Mapping):
-                yield self.load_from_dict(data)
-            elif isinstance(data, list):
-                for item in data:
-                    prepared_item = self.load_from_dict(item)
-                    if prepared_item is not None:
-                        yield prepared_item
+                for item in self.annotation_result_from_task(data):
+                    yield item
 
-    def load_from_dict(self, d):
-        has_annotations = 'completions' in d or 'annotations' in d
-        if not has_annotations and 'result' not in d:
-            raise KeyError('Each annotation dict item should contain "annotations" or "result" key, '
+            # many tasks
+            elif isinstance(data, list):
+                for task in data:
+                    for item in self.annotation_result_from_task(task):
+                        if item is not None:
+                            yield item
+
+    def annotation_result_from_task(self, task):
+        has_annotations = 'completions' in task or 'annotations' in task
+        if not has_annotations:
+            raise KeyError('Each task dict item should contain "annotations" or "completions" [deprecated],'
                            'where value is list of dicts')
-        result = []
-        if has_annotations:
-            # get last not skipped completion and make result from it
-            annotations = d.get('annotations')
-            if annotations is None:
-                annotations = d['completions']
-            tmp = list(filter(lambda x: not (x.get('skipped', False) or x.get('was_cancelled', False)), annotations))
-            if len(tmp) > 0:
-                result = sorted(tmp, key=lambda x: x.get('created_at', 0), reverse=True)[0]['result']
-            else:
-                return None
-            
-        elif 'result' in d:
-            result = d['result']
-        inputs = d['data']
-        outputs = defaultdict(list)
-        for r in result:
-            if 'from_name' in r and r['from_name'] in self._output_tags:
-                v = deepcopy(r['value'])
-                v['type'] = self._schema[r['from_name']]['type']
-                if 'original_width' in r:
-                    v['original_width'] = r['original_width']
-                if 'original_height' in r:
-                    v['original_height'] = r['original_height']
-                outputs[r['from_name']].append(v)
-        return {
-            'id': d['id'],
-            'input': inputs,
-            'output': outputs
-        }
+
+        # get last not skipped completion and make result from it
+        annotations = task['annotations'] if 'annotations' in task else task['completions']
+        
+        # skip cancelled annotations
+        cancelled = lambda x: not (x.get('skipped', False) or x.get('was_cancelled', False))
+        annotations = list(filter(cancelled, annotations))
+        if not annotations:
+            return None
+        
+        # sort by creation time
+        annotations = sorted(annotations, key=lambda x: x.get('created_at', 0), reverse=True)
+
+        for annotation in annotations:
+            inputs = task['data']
+            result = annotation['result']
+            outputs = defaultdict(list)
+
+            # get results only as output
+            for r in result:
+                if 'from_name' in r and r['from_name'] in self._output_tags:
+                    v = deepcopy(r['value'])
+                    v['type'] = self._schema[r['from_name']]['type']
+                    if 'original_width' in r:
+                        v['original_width'] = r['original_width']
+                    if 'original_height' in r:
+                        v['original_height'] = r['original_height']
+                    outputs[r['from_name']].append(v)
+
+            yield {
+                'id': task['id'],
+                'input': inputs,
+                'output': outputs,
+                'completed_by': annotation.get('completed_by', {}),
+                'annotation_id': annotation.get('id')
+            }
 
     def _check_format(self, fmt):
         pass
@@ -301,12 +319,15 @@ class Converter(object):
         output_file = os.path.join(output_dir, 'result.json')
         records = []
         item_iterator = self.iter_from_dir if is_dir else self.iter_from_json_file
+
         for item in item_iterator(input_data):
             record = deepcopy(item['input'])
             if item.get('id') is not None:
                 record['id'] = item['id']
             for name, value in item['output'].items():
                 record[name] = self._prettify(value)
+            record['annotator'] = item['completed_by'].get('email')
+            record['annotation_id'] = item['annotation_id']
             records.append(record)
 
         with io.open(output_file, mode='w', encoding='utf8') as fout:
@@ -318,6 +339,7 @@ class Converter(object):
         output_file = os.path.join(output_dir, 'result.csv')
         records = []
         item_iterator = self.iter_from_dir if is_dir else self.iter_from_json_file
+
         for item in item_iterator(input_data):
             record = deepcopy(item['input'])
             if item.get('id') is not None:
@@ -325,6 +347,8 @@ class Converter(object):
             for name, value in item['output'].items():
                 pretty_value = self._prettify(value)
                 record[name] = pretty_value if isinstance(pretty_value, str) else json.dumps(pretty_value)
+            record['annotator'] = item['completed_by'].get('email')
+            record['annotation_id'] = item['annotation_id']
             records.append(record)
 
         pd.DataFrame.from_records(records).to_csv(output_file, index=False, **kwargs)
@@ -367,7 +391,8 @@ class Converter(object):
             image_path = item['input'][data_key]
             if not os.path.exists(image_path):
                 try:
-                    image_path = download(image_path, output_image_dir, project_dir=self.project_dir, return_relative_path=True)
+                    image_path = download(image_path, output_image_dir, project_dir=self.project_dir,
+                                          return_relative_path=True, upload_dir=self.upload_dir)
                 except:
                     logger.error('Unable to download {image_path}. The item {item} will be skipped'.format(
                         image_path=image_path, item=item
@@ -418,7 +443,7 @@ class Converter(object):
                         'bbox': [x, y, w, h],
                         'ignore': 0,
                         'iscrowd': 0,
-                        'area': w * h
+                        'area': w * h,
                     })
                 elif "polygonlabels" in label:
                     points_abs = [(x / 100 * width, y / 100 * height) for x, y in label["points"]]
@@ -437,6 +462,9 @@ class Converter(object):
                 else:
                     raise ValueError("Unknown label type")
 
+                if os.getenv('LABEL_STUDIO_FORCE_ANNOTATOR_EXPORT'):
+                    annotations[-1].update({'annotator': item['completed_by'].get('email')})
+
         with io.open(output_file, mode='w', encoding='utf8') as fout:
             json.dump({
                 'images': images,
@@ -512,7 +540,9 @@ class Converter(object):
             create_child_node(doc, 'annotation', 'COCO2017', source_node)
             create_child_node(doc, 'image', 'flickr', source_node)
             create_child_node(doc, 'flickrid', 'NULL', source_node)
+            create_child_node(doc, 'annotator', item['completed_by'].get('email', 'none'), source_node)
             root_node.appendChild(source_node)
+
             owner_node = doc.createElement('owner')
             create_child_node(doc, 'flickrid', 'NULL', owner_node)
             create_child_node(doc, 'name', 'Label Studio', owner_node)
@@ -524,7 +554,6 @@ class Converter(object):
             root_node.appendChild(size_node)
             create_child_node(doc, 'segmented', '0', root_node)
 
-
             for bbox in bboxes:
                 name = bbox['rectanglelabels'][0]
                 x = int(bbox['x'] / 100 * width)
@@ -542,6 +571,7 @@ class Converter(object):
                 create_child_node(doc, 'ymin', str(y), bndbox_node)
                 create_child_node(doc, 'xmax', str(x + w), bndbox_node)
                 create_child_node(doc, 'ymax', str(y + h), bndbox_node)
+
                 object_node.appendChild(bndbox_node)
                 root_node.appendChild(object_node)
 
