@@ -6,9 +6,13 @@ import hashlib
 import logging
 import urllib
 import numpy as np
+import wave
+import shutil
 
 from operator import itemgetter
 from PIL import Image
+from urllib.parse import urlparse
+from nltk.tokenize import WhitespaceTokenizer
 
 
 logger = logging.getLogger(__name__)
@@ -27,30 +31,41 @@ def tokenize(text):
 
 
 def create_tokens_and_tags(text, spans):
-    tokens_and_idx = tokenize(text)
+    #tokens_and_idx = tokenize(text) # This function doesn't work properly if text contains multiple whitespaces...
+    token_index_tuples = [token for token in WhitespaceTokenizer().span_tokenize(text)]
+    tokens_and_idx = [(text[start:end], start) for start, end in token_index_tuples]
     if spans:
         spans = list(sorted(spans, key=itemgetter('start')))
         span = spans.pop(0)
+        span_start = span['start']
+        span_end = span['end']-1
         prefix = 'B-'
         tokens, tags = [], []
         for token, token_start in tokens_and_idx:
             tokens.append(token)
-            token_end = token_start + len(token) - 1
-            if not span or token_end < span['start']:
-                tags.append('O')
-            elif token_start >= span['end']:
-                # this could happen if prev label ends with whitespaces, e.g. "cat " "too"
-                # TODO: it is not right choice to place empty tag here in case when current token is covered by next span  # noqa
-                tags.append('O')
-            else:
-                tags.append(prefix + span['labels'][0])
-                if (span['end'] - 1) > token_end:
-                    prefix = 'I-'
-                elif len(spans):
+            token_end = token_start + len(token) #"- 1" - This substraction is wrong. token already uses the index E.g. "Hello" is 0-4
+            token_start_ind = token_start  #It seems like the token start is too early.. for whichever reason
+
+            #if for some reason end of span is missed.. pop the new span (Which is quite probable due to this method)
+            #Attention it seems like span['end'] is the index of first char afterwards. In case the whitespace is part of the
+            #labell we need to subtract one. Otherwise next token won't trigger the span update.. only the token after next..
+            if token_start_ind > span_end:
+                while spans:
                     span = spans.pop(0)
+                    span_start = span['start']
+                    span_end = span['end'] - 1
                     prefix = 'B-'
-                else:
-                    span = None
+                    if token_start <= span_end:
+                        break
+
+
+            if not span or token_end < span_start:
+                tags.append('O')
+            elif span_start <= token_end and span_end >= token_start_ind:
+                tags.append(prefix + span['labels'][0])
+                prefix = 'I-'
+            else:
+                tags.append('O')
     else:
         tokens = [token for token, _ in tokens_and_idx]
         tags = ['O'] * len(tokens)
@@ -58,20 +73,47 @@ def create_tokens_and_tags(text, spans):
     return tokens, tags
 
 
-def download(url, output_dir, filename=None):
-    if url.startswith('/data/upload/'):
-        url = 'http://localhost' + url
+def _get_upload_dir(project_dir=None, upload_dir=None):
+    """Return either upload_dir, or path by LS_UPLOAD_DIR, or project_dir/upload"""
+    if upload_dir:
+        return upload_dir
+    upload_dir = os.environ.get('LS_UPLOAD_DIR')
+    if not upload_dir and project_dir:
+        upload_dir = os.path.join(project_dir, 'upload')
+        if not os.path.exists(upload_dir):
+            upload_dir = None
+    if not upload_dir:
+        raise FileNotFoundError("Can't find upload dir: either LS_UPLOAD_DIR or project should be passed to converter")
+    return upload_dir
 
-    if '/data/' in url and '?d=' in url:
+
+def download(url, output_dir, filename=None, project_dir=None, return_relative_path=False, upload_dir=None):
+    is_local_file = url.startswith('/data/') and '?d=' in url
+    is_uploaded_file = url.startswith('/data/upload')
+
+    if is_uploaded_file:
+        upload_dir = _get_upload_dir(project_dir, upload_dir)
+        filename = url.replace('/data/upload/', '')
+        filepath = os.path.join(upload_dir, filename)
+        logger.debug('Copy {filepath} to {output_dir}'.format(filepath=filepath, output_dir=output_dir))
+        shutil.copy(filepath, output_dir)
+        if return_relative_path:
+            return os.path.join(os.path.basename(output_dir), filename)
+        return filepath
+
+    if is_local_file:
         filename, dir_path = url.split('/data/', 1)[-1].split('?d=')
         dir_path = str(urllib.parse.unquote(dir_path))
         if not os.path.exists(dir_path):
             raise FileNotFoundError(dir_path)
         filepath = os.path.join(dir_path, filename)
-        return filepath, False
+        if return_relative_path:
+            raise NotImplementedError()
+        return filepath
 
     if filename is None:
-        filename = hashlib.md5(url.encode()).hexdigest()
+        basename, ext = os.path.splitext(os.path.basename(urlparse(url).path))
+        filename = basename + '_' + hashlib.md5(url.encode()).hexdigest()[:4] + ext
     filepath = os.path.join(output_dir, filename)
     if not os.path.exists(filepath):
         logger.info('Download {url} to {filepath}'.format(url=url, filepath=filepath))
@@ -79,7 +121,9 @@ def download(url, output_dir, filename=None):
         r.raise_for_status()
         with io.open(filepath, mode='wb') as fout:
             fout.write(r.content)
-    return filepath, True
+    if return_relative_path:
+        return os.path.join(os.path.basename(output_dir), filename)
+    return filepath
 
 
 def get_image_size(image_path):
@@ -93,6 +137,11 @@ def get_image_size_and_channels(image_path):
     return w, h, c
 
 
+def get_audio_duration(audio_path):
+    with wave.open(audio_path, mode='r') as f:
+        return f.getnframes() / float(f.getframerate())
+
+
 def ensure_dir(dir_path):
     if not os.path.exists(dir_path):
         os.makedirs(dir_path)
